@@ -2,10 +2,10 @@
 # -*- coding: utf-8 -*-
 # vi:ts=4:et
 
+from . import localhost
 import pycurl
+import pytest
 import unittest
-import nose.plugins.attrib
-import nose.plugins.skip
 
 from . import util
 
@@ -73,7 +73,7 @@ class OptionConstantsTest(unittest.TestCase):
     @util.min_libcurl(7, 19, 4)
     def test_noproxy_setopt(self):
         curl = pycurl.Curl()
-        curl.setopt(curl.NOPROXY, 'localhost')
+        curl.setopt(curl.NOPROXY, localhost)
         curl.close()
 
     # CURLOPT_PROTOCOLS was introduced in libcurl-7.19.4
@@ -99,7 +99,7 @@ class OptionConstantsTest(unittest.TestCase):
 
     # CURLOPT_SOCKS5_GSSAPI_SERVICE was introduced in libcurl-7.19.4
     @util.min_libcurl(7, 19, 4)
-    @nose.plugins.attrib.attr('gssapi')
+    @pytest.mark.gssapi
     def test_socks5_gssapi_service_setopt(self):
         curl = pycurl.Curl()
         curl.setopt(curl.SOCKS5_GSSAPI_SERVICE, 'helloworld')
@@ -107,7 +107,7 @@ class OptionConstantsTest(unittest.TestCase):
 
     # CURLOPT_SOCKS5_GSSAPI_NEC was introduced in libcurl-7.19.4
     @util.min_libcurl(7, 19, 4)
-    @nose.plugins.attrib.attr('gssapi')
+    @pytest.mark.gssapi
     def test_socks5_gssapi_nec_setopt(self):
         curl = pycurl.Curl()
         curl.setopt(curl.SOCKS5_GSSAPI_NEC, True)
@@ -150,7 +150,7 @@ class OptionConstantsTest(unittest.TestCase):
         curl.close()
 
     @util.min_libcurl(7, 22, 0)
-    @nose.plugins.attrib.attr('gssapi')
+    @pytest.mark.gssapi
     def test_gssapi_delegation_options(self):
         curl = pycurl.Curl()
         curl.setopt(curl.GSSAPI_DELEGATION, curl.GSSAPI_DELEGATION_FLAG)
@@ -163,9 +163,16 @@ class OptionConstantsTest(unittest.TestCase):
     def test_sslversion_options(self):
         curl = pycurl.Curl()
         curl.setopt(curl.SSLVERSION, curl.SSLVERSION_DEFAULT)
+        curl.setopt(curl.SSLVERSION, curl.SSLVERSION_TLSv1)
+        curl.close()
+
+    # SSLVERSION_SSLv* return CURLE_BAD_FUNCTION_ARGUMENT with curl-7.77.0
+    @util.removed_in_libcurl(7, 77, 0)
+    @util.only_ssl
+    def test_legacy_sslversion_options(self):
+        curl = pycurl.Curl()
         curl.setopt(curl.SSLVERSION, curl.SSLVERSION_SSLv2)
         curl.setopt(curl.SSLVERSION, curl.SSLVERSION_SSLv3)
-        curl.setopt(curl.SSLVERSION, curl.SSLVERSION_TLSv1)
         curl.close()
 
     @util.min_libcurl(7, 34, 0)
@@ -186,20 +193,21 @@ class OptionConstantsTest(unittest.TestCase):
         curl.close()
 
     @util.min_libcurl(7, 43, 0)
-    @nose.plugins.attrib.attr('gssapi')
+    @pytest.mark.gssapi
     def test_proxy_service_name(self):
         curl = pycurl.Curl()
         curl.setopt(curl.PROXY_SERVICE_NAME, 'fakehttp')
         curl.close()
 
     @util.min_libcurl(7, 43, 0)
-    @nose.plugins.attrib.attr('gssapi')
+    @pytest.mark.gssapi
     def test_service_name(self):
         curl = pycurl.Curl()
         curl.setopt(curl.SERVICE_NAME, 'fakehttp')
         curl.close()
 
     @util.min_libcurl(7, 39, 0)
+    @util.only_ssl
     def test_pinnedpublickey(self):
         curl = pycurl.Curl()
         curl.setopt(curl.PINNEDPUBLICKEY, '/etc/publickey.der')
@@ -211,6 +219,7 @@ class OptionConstantsTest(unittest.TestCase):
         curl.setopt(curl.WILDCARDMATCH, '*')
         curl.close()
 
+    @util.only_unix
     @util.min_libcurl(7, 40, 0)
     def test_unix_socket_path(self):
         curl = pycurl.Curl()
@@ -218,19 +227,21 @@ class OptionConstantsTest(unittest.TestCase):
         curl.close()
 
     @util.min_libcurl(7, 36, 0)
+    @pytest.mark.http2
     def test_ssl_enable_alpn(self):
         curl = pycurl.Curl()
         curl.setopt(curl.SSL_ENABLE_ALPN, 1)
         curl.close()
 
     @util.min_libcurl(7, 36, 0)
+    @pytest.mark.http2
     def test_ssl_enable_npn(self):
         curl = pycurl.Curl()
         curl.setopt(curl.SSL_ENABLE_NPN, 1)
         curl.close()
 
     @util.min_libcurl(7, 42, 0)
-    @util.only_ssl_backends('nss')
+    @util.only_ssl_backends('nss', 'secure-transport')
     def test_ssl_falsestart(self):
         curl = pycurl.Curl()
         curl.setopt(curl.SSL_FALSESTART, 1)
@@ -252,12 +263,217 @@ class OptionConstantsTest(unittest.TestCase):
         curl.setopt(curl.ISSUERCERT, '/bogus-issuercert')
         curl.close()
 
-    @util.only_ssl
+    @util.only_ssl_backends('openssl', 'gnutls', 'nss')
     def test_capath(self):
         curl = pycurl.Curl()
         curl.setopt(curl.CAPATH, '/bogus-capath')
         curl.close()
 
+    @util.min_libcurl(7, 77, 0)
+    @util.only_ssl_backends('openssl')
+    def test_cainfo_blob(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.CAINFO_BLOB, 'bogus-cainfo-blob-as-str')
+        curl.setopt(curl.CAINFO_BLOB, None)
+        curl.setopt(curl.CAINFO_BLOB, b'bogus-cainfo-blob-as-bytes')
+        curl.unsetopt(curl.CAINFO_BLOB)
+        curl.close()
+
+    @util.min_libcurl(7, 71, 0)
+    @util.only_ssl
+    def test_sslcert_blob(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.SSLCERT_BLOB, 'bogus-sslcert-blob-as-str')
+        curl.setopt(curl.SSLCERT_BLOB, None)
+        curl.setopt(curl.SSLCERT_BLOB, b'bogus-sslcert-blob-as-bytes')
+        curl.unsetopt(curl.SSLCERT_BLOB)
+        curl.close()
+
+    @util.min_libcurl(7, 71, 0)
+    @util.only_ssl
+    def test_sslkey_blob(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.SSLKEY_BLOB, 'bogus-sslkey-blob-as-str')
+        curl.setopt(curl.SSLKEY_BLOB, None)
+        curl.setopt(curl.SSLKEY_BLOB, b'bogus-sslkey-blob-as-bytes')
+        curl.unsetopt(curl.SSLKEY_BLOB)
+        curl.close()
+
+    @util.min_libcurl(7, 71, 0)
+    @util.only_ssl
+    def test_issuercert_blob(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.ISSUERCERT_BLOB, 'bogus-issuercert-blob-as-str')
+        curl.setopt(curl.ISSUERCERT_BLOB, None)
+        curl.setopt(curl.ISSUERCERT_BLOB, b'bogus-issuercert-blob-as-bytes')
+        curl.unsetopt(curl.ISSUERCERT_BLOB)
+        curl.close()
+
+    # CURLOPT_PROXY_CAPATH was introduced in libcurl-7.52.0
+    @util.min_libcurl(7, 52, 0)
+    @util.only_ssl_backends('openssl', 'gnutls', 'nss')
+    def test_proxy_capath(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_CAPATH, '/bogus-capath')
+        curl.close()
+
+    @util.min_libcurl(7, 77, 0)
+    @util.only_ssl_backends('openssl')
+    def test_proxy_cainfo_blob(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_CAINFO_BLOB, 'bogus-cainfo-blob-as-str')
+        curl.setopt(curl.PROXY_CAINFO_BLOB, None)
+        curl.setopt(curl.PROXY_CAINFO_BLOB, b'bogus-cainfo-blob-as-bytes')
+        curl.unsetopt(curl.PROXY_CAINFO_BLOB)
+        curl.close()
+
+    @util.min_libcurl(7, 52, 0)
+    @util.only_ssl
+    def test_proxy_crlfile(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_CRLFILE, '/bogus-crlfile')
+        curl.close()
+
+    @util.min_libcurl(7, 52, 0)
+    @util.only_ssl
+    def test_proxy_sslcert(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_SSLCERT, '/bogus-sslcert')
+        curl.close()
+
+    @util.min_libcurl(7, 71, 0)
+    @util.only_ssl
+    def test_proxy_sslcert_blob(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_SSLCERT_BLOB, 'bogus-sslcert-blob-as-str')
+        curl.setopt(curl.PROXY_SSLCERT_BLOB, None)
+        curl.setopt(curl.PROXY_SSLCERT_BLOB, b'bogus-sslcert-blob-as-bytes')
+        curl.unsetopt(curl.PROXY_SSLCERT_BLOB)
+        curl.close()
+
+    @util.min_libcurl(7, 52, 0)
+    @util.only_ssl
+    def test_proxy_sslcerttype(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_SSLCERTTYPE, 'PEM')
+        curl.close()
+
+    @util.min_libcurl(7, 52, 0)
+    @util.only_ssl
+    def test_proxy_sslkey(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_SSLKEY, '/bogus-sslkey')
+        curl.close()
+
+    @util.min_libcurl(7, 71, 0)
+    @util.only_ssl
+    def test_proxy_sslkey_blob(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_SSLKEY_BLOB, 'bogus-sslkey-blob-as-str')
+        curl.setopt(curl.PROXY_SSLKEY_BLOB, None)
+        curl.setopt(curl.PROXY_SSLKEY_BLOB, b'bogus-sslkey-blob-as-bytes')
+        curl.unsetopt(curl.PROXY_SSLKEY_BLOB)
+        curl.close()
+
+    @util.min_libcurl(7, 52, 0)
+    @util.only_ssl
+    def test_proxy_sslkeytype(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_SSLKEYTYPE, 'PEM')
+        curl.close()
+
+    @util.min_libcurl(7, 71, 0)
+    @util.only_ssl
+    def test_proxy_issuercert_blob(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_ISSUERCERT_BLOB, 'bogus-issuercert-blob-as-str')
+        curl.setopt(curl.PROXY_ISSUERCERT_BLOB, None)
+        curl.setopt(curl.PROXY_ISSUERCERT_BLOB, b'bogus-issuercert-blob-as-bytes')
+        curl.unsetopt(curl.PROXY_ISSUERCERT_BLOB)
+        curl.close()
+
+    @util.min_libcurl(7, 52, 0)
+    @util.only_ssl
+    def test_proxy_keypasswd(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_KEYPASSWD, 'secret')
+        curl.close()
+
+    @util.min_libcurl(7, 52, 0)
+    @util.only_ssl
+    def test_proxy_ssl_verifypeer(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_SSL_VERIFYPEER, 1)
+        curl.close()
+
+    @util.min_libcurl(7, 52, 0)
+    @util.only_ssl
+    def test_proxy_ssl_verifyhost(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_SSL_VERIFYHOST, 2)
+        curl.close()
+
+    @util.min_libcurl(7, 52, 0)
+    @util.only_ssl
+    def test_proxy_pinnedpublickey(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_PINNEDPUBLICKEY, '/etc/publickey.der')
+        curl.close()
+
+    @util.min_libcurl(7, 52, 0)
+    @util.only_ssl
+    def test_proxy_sslversion_options(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_SSLVERSION, curl.SSLVERSION_DEFAULT)
+        curl.setopt(curl.PROXY_SSLVERSION, curl.SSLVERSION_TLSv1)
+        curl.setopt(curl.PROXY_SSLVERSION, curl.SSLVERSION_TLSv1_0)
+        curl.setopt(curl.PROXY_SSLVERSION, curl.SSLVERSION_TLSv1_1)
+        curl.setopt(curl.PROXY_SSLVERSION, curl.SSLVERSION_TLSv1_2)
+        curl.close()
+ 
+    # SSLVERSION_SSLv* return CURLE_BAD_FUNCTION_ARGUMENT with curl-7.77.0
+    @util.min_libcurl(7, 52, 0)
+    @util.removed_in_libcurl(7, 77, 0)
+    @util.only_ssl
+    def test_legacy_proxy_sslversion_options(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_SSLVERSION, curl.SSLVERSION_SSLv2)
+        curl.setopt(curl.PROXY_SSLVERSION, curl.SSLVERSION_SSLv3)
+        curl.close()
+
+    @util.min_libcurl(7, 52, 0)
+    @util.only_ssl
+    def test_proxy_ssl_cipher_list(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_SSL_CIPHER_LIST, 'RC4-SHA:SHA1+DES')
+        curl.close()
+
+    @util.min_libcurl(7, 52, 0)
+    @util.only_ssl
+    def test_proxy_ssl_options(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_SSL_OPTIONS, curl.SSLOPT_ALLOW_BEAST)
+        curl.setopt(curl.PROXY_SSL_OPTIONS, curl.SSLOPT_NO_REVOKE)
+        curl.close()
+
+    # Apparently TLSAUTH_TYPE=SRP is an unknown option on appveyor
+    @util.only_unix
+    @util.min_libcurl(7, 52, 0)
+    @util.only_ssl_backends('openssl', 'gnutls', 'schannel')
+    def test_proxy_tlsauth(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_TLSAUTH_TYPE, "SRP")
+        curl.setopt(curl.PROXY_TLSAUTH_USERNAME, "test")
+        curl.setopt(curl.PROXY_TLSAUTH_PASSWORD, "test")
+        curl.close()
+
+    @util.min_libcurl(7, 71, 0)
+    @util.only_ssl
+    def test_proxy_issuercert(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_ISSUERCERT, '/bogus-issuercert')
+        curl.close()
+
     @util.only_ssl
     def test_crlfile(self):
         curl = pycurl.Curl()
@@ -270,7 +486,7 @@ class OptionConstantsTest(unittest.TestCase):
         curl.setopt(curl.RANDOM_FILE, '/bogus-random')
         curl.close()
 
-    @util.only_ssl_backends('openssl', 'gnutls')
+    @util.only_ssl_backends('openssl', 'gnutls', 'secure-transport', 'schannel')
     def test_egdsocket(self):
         curl = pycurl.Curl()
         curl.setopt(curl.EGDSOCKET, '/bogus-egdsocket')
@@ -312,6 +528,38 @@ class OptionConstantsTest(unittest.TestCase):
         curl.setopt(curl.SSL_OPTIONS, curl.SSLOPT_NO_REVOKE)
         curl.close()
 
+    @util.min_libcurl(7, 55, 0)
+    def test_request_target_option(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.REQUEST_TARGET, '*')
+        curl.close()
+
+    @util.min_libcurl(7, 64, 0)
+    def test_http09_allowed_option(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.HTTP09_ALLOWED, 1)
+        curl.close()
+
+    @util.min_libcurl(7, 61, 0)
+    @util.only_ssl_backends('openssl', 'schannel')
+    def test_tls13_ciphers(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.TLS13_CIPHERS, 'TLS_CHACHA20_POLY1305_SHA256')
+        curl.close()
+
+    @util.min_libcurl(7, 61, 0)
+    @util.only_ssl_backends('openssl', 'schannel')
+    def test_proxy_tls13_ciphers(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.PROXY_TLS13_CIPHERS, 'TLS_CHACHA20_POLY1305_SHA256')
+        curl.close()
+
+    @util.min_libcurl(7, 75, 0)
+    def test_aws_sigv4(self):
+        curl = pycurl.Curl()
+        curl.setopt(curl.AWS_SIGV4, 'provider1:provider2')
+        curl.close()
+
 class OptionConstantsSettingTest(unittest.TestCase):
     def setUp(self):
         self.curl = pycurl.Curl()
@@ -332,6 +580,7 @@ class OptionConstantsSettingTest(unittest.TestCase):
     def test_keypasswd(self):
         self.curl.setopt(self.curl.KEYPASSWD, 'secret')
 
+    @util.only_telnet
     def test_telnetoptions(self):
         self.curl.setopt(self.curl.TELNETOPTIONS, ('TTYPE=1', 'XDISPLOC=2'))
 
@@ -394,21 +643,31 @@ class OptionConstantsSettingTest(unittest.TestCase):
         self.curl.setopt(self.curl.HTTP_VERSION, self.curl.CURL_HTTP_VERSION_1_0)
         self.curl.setopt(self.curl.HTTP_VERSION, self.curl.CURL_HTTP_VERSION_1_1)
 
-    @nose.plugins.attrib.attr('http2')
     @util.min_libcurl(7, 33, 0)
+    @util.only_http2
     def test_http_version_2_0(self):
         self.curl.setopt(self.curl.HTTP_VERSION, self.curl.CURL_HTTP_VERSION_2_0)
 
-    @nose.plugins.attrib.attr('http2')
     @util.min_libcurl(7, 43, 0)
+    @util.only_http2
     def test_http_version_2(self):
         self.curl.setopt(self.curl.HTTP_VERSION, self.curl.CURL_HTTP_VERSION_2)
 
-    @nose.plugins.attrib.attr('http2')
     @util.min_libcurl(7, 47, 0)
+    @util.only_http2
     def test_http_version_2tls(self):
         self.curl.setopt(self.curl.HTTP_VERSION, self.curl.CURL_HTTP_VERSION_2TLS)
 
+    @util.min_libcurl(7, 49, 0)
+    @util.only_http2
+    def test_http_version_2prior_knowledge(self):
+        self.curl.setopt(self.curl.HTTP_VERSION, self.curl.CURL_HTTP_VERSION_2_PRIOR_KNOWLEDGE)
+
+    @util.min_libcurl(7, 66, 0)
+    @util.only_http3
+    def test_http_version_3(self):
+        self.curl.setopt(self.curl.HTTP_VERSION, self.curl.CURL_HTTP_VERSION_3)
+
     @util.min_libcurl(7, 21, 5)
     def test_sockopt_constants(self):
         assert self.curl.SOCKOPT_OK is not None
@@ -420,8 +679,10 @@ class OptionConstantsSettingTest(unittest.TestCase):
         assert self.curl.PROTO_SMB is not None
         assert self.curl.PROTO_SMBS is not None
 
+    # Apparently TLSAUTH_TYPE=SRP is an unknown option on appveyor
+    @util.only_unix
     @util.min_libcurl(7, 21, 4)
-    @util.only_ssl_backends('openssl', 'gnutls')
+    @util.only_ssl_backends('openssl', 'gnutls', 'schannel')
     def test_tlsauth(self):
         self.curl.setopt(self.curl.TLSAUTH_TYPE, "SRP")
         self.curl.setopt(self.curl.TLSAUTH_USERNAME, "test")
@@ -446,3 +707,6 @@ class OptionConstantsSettingTest(unittest.TestCase):
     @util.min_libcurl(7, 33, 0)
     def test_xauth_bearer(self):
         self.curl.setopt(self.curl.XOAUTH2_BEARER, 'test')
+        
+    def test_cookielist_constants(self):
+        self.assertEqual(pycurl.OPT_COOKIELIST, pycurl.COOKIELIST)
